/*
	Spellcast

	Copyright (c) 2014 - 2020 Cédric Ronvel

	The MIT License (MIT)

	Permission is hereby granted, free of charge, to any person obtaining a copy
	of this software and associated documentation files (the "Software"), to deal
	in the Software without restriction, including without limitation the rights
	to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
	copies of the Software, and to permit persons to whom the Software is
	furnished to do so, subject to the following conditions:

	The above copyright notice and this permission notice shall be included in all
	copies or substantial portions of the Software.

	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
	AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
	LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
	OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
	SOFTWARE.
*/

"use strict" ;



require( './patches.js' ) ;
var fs = require( 'fs' ) ;
var term = require( 'terminal-kit' ).terminal ;



async function summonMakefile( book , spellcastBin ) {
	var ruleList , npmInstall , makefile = '' , makefileFile = 'Makefile' ;

	// Argument management
	if ( typeof spellcastBin !== 'string' ) {
		npmInstall = true ;
		spellcastBin = './node_modules/spellcast/bin/spellcast' ;
	}


	// First, advertise...
	makefile += '\n# This Makefile has been automatically generated by Spellcast.\n# See: https://www.npmjs.org/package/spellcast\n\n' ;

	// Get all rules
	ruleList = Object.keys( book.spells ) ;

	Object.keys( book.summonings ).forEach( summoning => {
		// Do not copy summoning that share the same key as a spellcasting.
		if ( book.spells[ summoning ] || summoning === makefileFile ) { return ; }

		// Do not copy non-static summoning.
		// /!\ Some glob-summoning are possible to write in Makefile using the '%' char /!\
		if ( book.summonings[ summoning ].type !== 'static' ) { return ; }

		ruleList.push( summoning ) ;
	} ) ;

	// Build all rules
	ruleList.forEach( rule => {
		makefile += rule + ':' ;
		if ( npmInstall ) { makefile +=	' ' + spellcastBin ; }
		makefile += '\n\t' + spellcastBin + ' ' + rule + '\n\n' ;
	} ) ;

	// If necessary, add the rule to install spellcast locally
	if ( npmInstall ) {
		makefile +=	spellcastBin + ':\n\tnpm install spellcast\n\n' ;
	}

	// Make every rules 'PHONY' rules
	if ( ruleList.length ) {
		makefile +=
			"# Make every rules 'PHONY' rules, let Spellcast handle dependencies and everything else.\n" +
			'.PHONY: ' + ruleList.join( ' ' ) ;
	}

	makefile += '\n\n' ;

	try {
		await fs.writeFileAsync( makefileFile , makefile ) ;
	}
	catch ( error ) {
		term.red( 'Cannot create the file ' ).italic.bold.red( makefileFile + '\n' ) ;
		throw error ;
	}
}

module.exports = summonMakefile ;

